/*
 * Este fichero contiene funciones básicas de consulta/introducción SQL y cálculo
 * relacionadas con la consulta de canales comerciales disponibles por parte del
 * usuario. 
 * 
 * Algunos cálculos requieren funciones matemáticas muy concretas que hemos 
 * implementado en MathTools.js
 */
var MathTools = require('./MathTools.js');
var mysql = require('mysql');

// Creamos una conexión a la BBDD con los siguientes parámetros y credenciales.
var satinfoDB = mysql.createConnection(
    {
        host    :   'localhost',
        user    :   'consultor',
        password:   'contrasena',
        database:   'SatInfo'
    }    
);
satinfoDB.connect();

/* Ejemplo de uso
var orden = "INSERT INTO Canal VALUES ('11200','17024','Da Vinci Learning','10758','V','Nagravision 3','T','HD',null,'27500','3/4','DVB-S2','QPSK');";
satinfoDB.query(orden, function(err, rows, fields){
    if(err) throw err;
    
    console.log("Exectuted 'SELECT * FROM Canal' ");
    console.log(rows);
    satinfoDB.end();
});
*/

exports.filtrarTranspondersPorIdioma = function(idiomasRequeridos, callback){
    /*
     * Esta función busca en toda la base de datos de transpondedores y guarda 
     * en el array 'listaFiltradaPorIdiomas' los TID de los transpondedores que 
     * ofrecen retransmisiones en alguno de los idiomas requeridos por el 
     * usuario ( idiomasRequeridos ).
     */  
    console.log("filtrarTranspondersPorIdioma()");
    
    // Construimos la sentencia SQL
    var orden = 'SELECT DISTINCT TID from CanalIdioma WHERE CanalIdioma.cod_iso="'+idiomasRequeridos[0]+'" OR CanalIdioma.cod_iso="'+idiomasRequeridos[1]+'" OR CanalIdioma.cod_iso="'+idiomasRequeridos[2]+'" OR CanalIdioma.cod_iso="'+idiomasRequeridos[3]+'";';
    // Hacemos efectiva la sentencia
    satinfoDB.query(orden, function(err, rows, fields){
        if(err) throw err;

        //console.log("Utilidades/filtrarTranspondersPorIdioma/rows: "+JSON.stringify(rows));
        callback(rows);
    });

};

exports.filtrarTranspondersPorHuella = function(ruta, antena, transpondersPotencialmenteUtiles, callback){
    /*
     * Esta función recibe una lista de TID en transpondedoresPotencialmenteUtiles, la cual filtra
     * atendiendo a si la ruta está contenida en la huella que describe el área donde se recibe la
     * potencia suficiente para que el sistema funcione utilizando la antena.
     */
    console.log("filtrarTranspondersPorHuella()");
    // Definimos la variable de respuesta donde almacenaremos el resultado del proceso
    var respuesta = [];
    
    // El siguiente bloque de código funciona como un pseudo bucle
    var max = transpondersPotencialmenteUtiles.length -1;   // Límite de iteraciones
    var cnt = 0;                                            // Variable que cuenta las iteraciones
    // Llamamos a la función que nos devuelve el área de cobertura para las antenas de tipo <antena> de un transpondedor
    getHuella(antena, transpondersPotencialmenteUtiles[cnt].TID, callbackDeGetHuella = function(huella, TIDenCuestion){
	// Sólo si tenemos las coordenadas que describen la huella, comprobamos que la ruta esté contenida en ella
	if(huella[0]){
        	console.log("filtrarTranspondersPorHuella/getHuella/huella: ("+TIDenCuestion+") "+JSON.stringify(huella));
        	if(estaRutaEnHuella(ruta, huella)){
         	   // Si la ruta está en la huella, añadimos el identificador del transpondedor a la variable de respuesta
         	   respuesta.push(TIDenCuestion);
         	   //console.log("filtrarTranspondersPorHuella/respuesta: "+respuesta);
      		}else{
	        console.log("El transpondedor no es visible en la ruta");
        	}
	}
        // Si cnt ha llegado al máximo de iteraciones el pseudobucle ha terminado y devolvemos la respuesta
        if((cnt) === max){
            console.log("CALLBACK()");
            callback(respuesta);
        }else{
            // Si no hemos terminado con el pseudobucle actualizamos la variable de iteraciones y...
            cnt++;
            //... llamamos a la función sobre la que hay que iterar
            getHuella(antena, transpondersPotencialmenteUtiles[cnt].TID, callbackDeGetHuella);
        }
        
    });
    
}

function getHuella(antena, transpondedorTID, callback){
    /*
     * Esta funcion devuelve las coordenadas que determinan el área de cobertura de transpondedorTID para el nivel de potencia
     * requerido por antena.
     */
    
    // Construimos la sentencia SQL que nos proporciona la posicion y el nombre de la huella del transpondedor en cuestión
    var orden1 = "SELECT posicion, nombre_huella FROM Transpondedor WHERE Transpondedor.TID='"+transpondedorTID+"';";
    //console.log("getHuella/orden1: "+orden1);
    // Hacemos efectiva la consulta que nos devuelve la posicion y el nombre de la huella
    satinfoDB.query(orden1, function(err, posYnombre, fields){
        // Construimos la sentencia SQL que nos dará las coordenadas que definen el área de cobertura a partir de la posición y el nombre de la huella
        var orden2 = "SELECT huella FROM Huella WHERE Huella.posicion='"+posYnombre[0].posicion+"' AND Huella.nombre_huella='"+posYnombre[0].nombre_huella+"';";
        //console.log("getHuella/orden2: "+orden2);
        // Hacemos efectiva la orden2
        satinfoDB.query(orden2, function(err, huellas, fields){
            if(err) throw err;
            
            // Comprobamos que la respuesta de la BBDD no esté vacía antes de continuar con el proceso
            if(!huellas[0]){
                console.log("No hay huellas guardadas con los parámetros solicitados.");
                callback([], transpondedorTID);
            }else{
                // Devolvemos el área de cobertura correspondiente al nivel de potencia determinado por antena
                switch(antena){
                case 'low':     callback(JSON.parse(huellas[0].huella).low, transpondedorTID);
                    //console.log("huella = "+JSON.stringify(JSON.parse(huellas[0].huella).low));
                    break;
                case 'medium':  callback(JSON.parse(huellas[0].huella).medium, transpondedorTID);
                    //console.log("huella = "+JSON.stringify(JSON.parse(huellas[0].huella).medium));
                    break;
                case 'high':    callback(JSON.parse(huellas[0].huella).high, transpondedorTID);
                    //console.log("huella = "+JSON.stringify(JSON.parse(huellas[0].huella).high));
                    break;
                }
            }
        });
    });
        
}

function estaRutaEnHuella(ruta, huella){
    /*
     * Esta funcion recorre los puntos que conforman la ruta y comprueba si todos 
     * están dentro de la huella interesante (el interés lo marcará la antena).
     * En caso de que todos los puntos estén dentro de la huella, la ruta lo está;
     * en cualquier otro caso, no.
     */
    console.log("estaRutaEnHuella()");
    
    // Definimos la variable que devuelve true si la ruta está dentro de la huella
    var loEsta = true;
    
    // Recorremos los puntos de la ruta para comprobar si están dentro de la huella
    for (var i = 0; i <= ruta.length-1; i++){
        //console.log("huella: "+JSON.stringify(huella));
        
        // En caso de que un punto de la ruta esté fuera de la huella, consideramos que la ruta al completo no está dentro
        if(!estaPuntoEnPoligono(ruta[i], huella)){
            loEsta = false;
        }
        //console.log("estaPuntoEnPoligono/loEsta: "+loEsta);
    }
    //console.log("estaRutaEnHuella/loEsta: "+loEsta);
    return loEsta;
}

function estaPuntoEnPoligono(punto, poligono){
    /*
     * Esta función determina si un punto está dentro de un polígono. Consultar la documentación
     * para una explicación en profundidad del algoritmo.
     */
    
    // vDespl será un array donde guardaremos vectores desplazamiento, desde el punto en cuestion a
    // los puntos que determinan el polígono
    var vDespl = [];
    for(var i = 0; i <= poligono.length-1; i++){
        // r es un vector desplazamiento
        var r = new Array(2);
        r[0] = poligono[i][0] - punto[0]; // Cálculo con las componentes x
        r[1] = poligono[i][1] - punto[1]; // Cálculo con las componentes y
        // Añadimos el vector desplazamiento al array donde los estamos guardando
        vDespl.push(r);
    }
    console.log("## vDespl", vDespl, "punto", punto, "polig", poligono);
    // Calcular y acumular diferencia angular entre vec. despl. consecutivos (MathTools.difAng(v1, v2))
    var acum = MathTools.difAng(vDespl[0], vDespl[vDespl.length-1]);
    for(var n = 1; n <= vDespl.length-1; n++){
        acum = acum + MathTools.difAng(vDespl[n], vDespl[n-1]);
        //console.log("acum: "+acum);
    }
    
    // Si acum acaba con un valor cercano a 0 significa que el punto está fuera del polígono
    if(Math.abs(acum)<0.001){
        return false;
    }else{
        return true;
    }
}

exports.getInfoDeCanales = function(listaTID, callback){
    /*
     * Esta función extrae de la base de datos de Canales todos los parámetros 
     * de configuración interesantes para la sintonización del canal televisivo.
     * 
     * En listaTID le pasamos un array con los TID (identificadores de transponder)
     * que nos resulten relevantes, de cuyos canales queramos obtener la 
     * información.
     * 
     */
    console.log("getInfoDeCanales()");
    
    // Definimos la variable de respuesta
    var respuesta = [];
    
    // Mientras i controla las iteraciones sincronas y genera todas las sentencias SQL,
    // cnt controla que se hayan añadido la informacion de todos los canales a respuesta
    var i = 0;
    
    consultarCanal(listaTID[i], callbackDeConsultarCanal = function(infoCanal){
        
        incluirNombreSatelite(infoCanal, function(infoCanalSat){
        
            incluirIdiomas(infoCanalSat, function(infoConIdiomas){

                // Añadimos la informacion del canal a la variable respuesta
                respuesta.push(infoConIdiomas);

                // Si ya se han obtenido todos los canales, hemos terminado
                if (i === listaTID.length-1) {
                    console.log('COMPLETED');
                    callback((respuesta[0]));   // La respuesta acaba contenida entre dos pares de corchetes, de ahí el índice 0
                    console.log(" # respuesta: "+JSON.stringify(respuesta[0]));
                }else{
                    consultarCanal(listaTID[i++], callbackDeConsultarCanal);
                } 
            });
        });
    });
}

function consultarCanal(TID, callback){
    /*
     * Esta funcion obtiene la información general de la tabla Canal en la fila correspondiente
     * a la TID con la que estamos trabajando.
     */
    var orden1 = 'SELECT * FROM Canal WHERE Canal.TID="'+TID+'";';
    satinfoDB.query(orden1, function(err, info, fields){
        if(err) throw err;

        callback(info);
    });
}

function incluirNombreSatelite(info, callback){
    /*
     * Esta funcion añade los idiomas recogidos de la consulta a CanalIdioma al 
     * objeto que contiene la información general del canal recogida de la consulta
     * a la tabla Canal.
     */
    
    // Definimos la variable de respuesta
    var nombreSatIncluidos = [];
    
    // Definimos la variable de iteración para nuestro pseudobucle
    var i = 0;
    
    // Recorremos info añadiendo en cada iteración sus idiomas a cada elemento de info
    consultarNombreSatelite(info[i], callbackconsultarNombreSatelite = function(infoConNombre){
        // Incluimos en el array de respuesta los elementos, con los idiomas ya incluidos
        nombreSatIncluidos.push(infoConNombre);
        
        // Si hemos terminado de recorrer info llamamos al callback y en caso contrario seguimos iterando
        if(i === info.length-1){
            callback(nombreSatIncluidos);
        }else{
            consultarNombreSatelite(info[i++], callbackconsultarNombreSatelite);
        }
    });
}

function consultarNombreSatelite(info1Canal, callback){
    /*
     * Esta función obtiene el nombre del satélite que alberga un transpondedor.
     */
    var orden = "SELECT nombre_sat FROM Transpondedor WHERE Transpondedor.TID='"+info1Canal.TID+"';";
    satinfoDB.query(orden, function(err, nombre_sat, fields){
        if(err) throw err;

        info1Canal["nombre_sat"] = nombre_sat[0].nombre_sat;
        callback(info1Canal);
    });
}

function incluirIdiomas(info, callback){
    /*
     * Esta funcion añade los idiomas recogidos de la consulta a CanalIdioma al 
     * objeto que contiene la información general del canal recogida de la consulta
     * a la tabla Canal.
     */
    
    // Definimos la variable de respuesta
    var idiomasIncluidos = [];
    
    // Definimos la variable de iteración para nuestro pseudobucle
    var i = 0;
    
    // Recorremos info añadiendo en cada iteración sus idiomas a cada elemento de info
    consultarCanalIdioma(info[i], callbackConsultarCanalIdioma = function(infoConIdiomas){
        // Incluimos en el array de respuesta los elementos, con los idiomas ya incluidos
        idiomasIncluidos.push(infoConIdiomas);
        
        // Si hemos terminado de recorrer info llamamos al callback y en caso contrario seguimos iterando
        if(i === info.length-1){
            callback(idiomasIncluidos);
        }else{
            consultarCanalIdioma(info[i++], callbackConsultarCanalIdioma);
        }
    });
}

function consultarCanalIdioma(info1Canal, callback){
    /*
     * Esta función recibe el objeto que contiene la información general de un Canal
     * y le añade un nuevo campo con los idiomas de ese canal.
     */
    var orden = 'SELECT cod_iso FROM CanalIdioma WHERE CanalIdioma.TID="'+info1Canal.TID+'" AND CanalIdioma.SID="'+info1Canal.SID+'";';
    satinfoDB.query(orden, function(err, idiomas, fields){
        if(err) throw err;
        
        // Preferimos simplificar la notación, enumerando los códigos iso en un array que será el valor para la clave "idiomas"
        var idiomasArray = [];
        for(var i = 0; i <= idiomas.length-1; i++)
            idiomasArray.push(idiomas[i].cod_iso);
        
        info1Canal["idiomas"] = idiomasArray;
        callback(info1Canal);
    });
}

exports.getIdiomasDisponibles = function(callback){
    /*
     * Esta funcion devuelve todos los idiomas disponibles que hay en la BBDD para
     * poder ofertárselos al usuario en los desplegables de la web.
     */
    console.log("Utilidades.getIdiomasDisponibles()");
    
    // Construimos la orden SQL
    var orden = 'SELECT DISTINCT cod_iso FROM CanalIdioma;';
    // Hacemos efectiva la orden SQL
    satinfoDB.query(orden, function(err, rows, fields){
        if(err) throw err;
        
        console.log("idiomasDisponibles/rows: "+JSON.stringify(rows));
        callback(rows);
    });
}
