var fs = require('fs');
var mysql = require('mysql');
var satinfoDB = mysql.createConnection(
    {
        host    :   'localhost',
        user    :   'actualizador',
        password:   'contrasena',
        database:   'SatInfo'
    }    
);
satinfoDB.connect();

/* Ejemplo de uso
var orden = "INSERT INTO Canal VALUES ('11200','17024','Da Vinci Learning','10758','V','Nagravision 3','T','HD',null,'27500','3/4','DVB-S2','QPSK');";
satinfoDB.query(orden, function(err, rows, fields){
    if(err) throw err;
    
    console.log("Exectuted 'SELECT * FROM Canal' ");
    console.log(rows);
    satinfoDB.end();
});
*/

/*
 * El siguiente bloque de codigo es la funcion principal que llama ordenadamente al resto
 * de funciones, auxiliares.
 */
borrarTablas(function(){
    guardarEnBBDD('./archivo/0130.csv', function(){
        console.log("BBDD actualizada. Pulse ctrl+C para salir del script.");

    });
});

function borrarTablas(callback){
    /*
     * Esta funcion vacia las tablas que vamos a rellenar de nuevo
     */
    console.log("borrarTablas();");
    
    var orden = "DELETE FROM Canal;";
    satinfoDB.query(orden, function(err, resultado){
        if(err) throw err;
        
        var orden2 = "DELETE FROM CanalIdioma";
        satinfoDB.query(orden2, function(err, resultado){
            if(err) throw err;
            
            var orden3 = "DELETE FROM Transpondedor";
            satinfoDB.query(orden3, function(err, resultado){
                if(err) throw err;
                
                callback(); 
            });
        });
        
    });
}

function guardarEnBBDD (pathCSV, callback){
    /*
     * Esta funcion es la que introduce en la BBDD los nuevos datos leidos del archivo csv.
     */
    console.log("guardarCanales()");
    
    importarCSV(pathCSV, function(objetoJSON){
        //console.log(JSON.stringify(objetoJSON));
        //console.log((objetoJSON[25].Encryption.split("|")));
        
        // Recorremos la respuesta de importarCSV para generar las sentencias SQL correspondiente a cada elemento de ella
        for(var i = 0; i<objetoJSON.length-1; i++){
            // Construimos la sentencia SQL que actua sobre la tabla Canal
            var orden1 = 'INSERT INTO Canal (localID, TID, SID, VPID, nombre, frecuencia, polaridad, encriptacion, tipo, resolucion, URL, SymbolRate, FEC, codificacion, modulacion, compresion)values ("'
                                                      +(i+1)+
                                                 '","'+objetoJSON[i].TID+
                                                 '","'+objetoJSON[i].SID+
                                                 '","'+objetoJSON[i].VPID+
                                                 '","'+objetoJSON[i].Channel+
                                                 '","'+objetoJSON[i].Frequency+
                                                 '","'+objetoJSON[i].Polarisation+
                                                 '","'+objetoJSON[i].Encryption.split("|")+
                                                 '","'+objetoJSON[i].Type+
                                                 '","'+objetoJSON[i].Resolution+
                                                 '","'+objetoJSON[i].URL+
                                                 '","'+objetoJSON[i].SR+
                                                 '","'+objetoJSON[i].FEC+
                                                 '","'+objetoJSON[i].Encoding+
                                                 '","'+objetoJSON[i].Modulation+
                                                 '","'+objetoJSON[i].Compression+'");';
            //console.log("Orden "+i+": " + orden1);
            
            // Hacemos efectiva la orden1
            satinfoDB.query(orden1, function(err, resultado){
                if(err) throw err;
                //console.log("affectedRows: "+JSON.stringify(resultado.affectedRows));
            });
            
            // Separamos los distintos idiomas y los guardamos en idiomas, que es un array
            var idiomas = objetoJSON[i].Language.split("|");
            
            // Recorremos el array idiomas...
            for (var n = 0; n <= idiomas.length-1; n++){
                //... generando la sentencia SQL correspondiente para guardar cada uno de los idiomas
                var orden2 = 'INSERT IGNORE INTO CanalIdioma values ("'
                                                      +objetoJSON[i].TID+
                                                 '","'+objetoJSON[i].SID+
                                                 '","'+objetoJSON[i].VPID+
                                                 '","'+idiomas[n]+'");';
                //console.log("ORDEN NUEVA: "+orden2);
                
                // Hacemos efectiva la sentencia que guarda los idiomas del Canal en cuestion
                satinfoDB.query(orden2, function(err, res){
                    if(err) throw err;
                });
            }
            
            // Construimos la sentencia SQL que guarda la informacion del transpondedor relativo al Canal en cuestion
            var orden3 = 'INSERT IGNORE INTO Transpondedor values ("'
                                                         +objetoJSON[i].TID+
                                                    '","'+objetoJSON[i].NORAD+
                                                    '","'+objetoJSON[i].Satellite+
                                                    '","'+objetoJSON[i].Position+
                                                    '","'+objetoJSON[i].Beam+'");';
            //console.log("orden3: "+orden3);
            
            // Hacemos efectiva la orden3
            satinfoDB.query(orden3, function(err, res){
                if(err) throw err;
            });
            
            // Si hemos procesado todos los elementos de objetoJSON ya esta actualizada la BBDD (el ultimo elemento no es de informacion, por eso lo denostamos)
            if(i===objetoJSON.length-2)
                callback();
        }
    });
};

importarCSV = function(path, callback){
    /*
     * Esta funcion lee el archivo de texto con formato csv y lo convierte en un objeto
     * JSON sobre el que podemos trabajar comodamente.
     */
    console.log("importarCSV");
    
    //leo el archivo 
    fs.readFile(path, 'UTF-8', function(err, archivoLeido){
	if(err) throw err;

        var archivoCSV = archivoLeido.toString();
        //guardo las líneas en un array
        var lineas = archivoCSV.split("\n");
        var resultado = [];
        //extraigo las claves de la primera línea
        var claves = lineas[0].split(";");
        //recorro las líneas separando sus campos
        for(var i= 1; i<lineas.length-1; i++){
            var obj = {};
            var lineaActual = lineas[i].split(";");
            //recorro los campos y voy guardando (clave, valor) en el objeto
            for(var j=0;j<claves.length;j++){
                obj[claves[j]] = lineaActual[j];
            }
            resultado.push(obj);
        }
        //console.log("Resultado: "+JSON.stringify(resultado));
        callback(resultado);
    });
};
