/* 
 * 
 */
var express = require('express');           //http://expressjs.com/4x/api.html
var elServidor = express();
var laLogica = require('./logica.js');

/*
 * La función elServidor.use() incluye la función de middleware que será ejecutada
 * cada vez que se reciba una petición, ya que no le hemos asignado path por el
 * cual ejecutarse.
 * expressjs.com/es/guide/using-middleware.html
 */
elServidor.use (function(req, res, next) {
    console.log("Estoy en elServidor.use(...)");
    // Permitimos el acceso desde cualquier origen
    res.header("Access-Control-Allow-Origin", "*");
    
    // Extraemos los datos del paquete recibido y los guardamos como cuerpo de la petición
    var data='';
    req.setEncoding('utf8');
    req.on('data', function(chunk) {
       data += chunk;
    });
    
    req.on('end', function() {
        req.body = data;
        console.log("req.body = "+ req.body);
        next(); // la petición continúa
    });
    
    
});

/*
 * Las funciones siguientes llaman a la funcion de logica.js que les corresponde
 * según la naturaleza de la petición REST que ha recibido el servidor. Por eso
 * atienden al método (GET/POST) y la url.
 * 
 * En todas ellas el método de elServidor concuerda con el método REST, el primer
 * parámetro corresponde a la url requerida y el segundo parámetro es la función
 * de callback donde se encuentra el código que debe ejecutarse una vez detectado
 * que la petición del cliente va dirigida a esa función en particular.
 * 
 * La función de callback tiene dos parámetros: el objeto petición de cliente (req)
 * y el objeto respuesta del servidor (res).
 */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  
 *   Funcion que maneja las peticiones POST /canales/          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
elServidor.post('/canales/', function(req, res){
    console.log('Recibi peticion POST /canales/');
      
    laLogica.obtenerCanalesDisponibles(JSON.parse(req.body), function(respuesta){
        console.log("Hemos llamado a laLogica.obtenerCanalesDisponibles() con éxito");
        console.log("respuesta server: "+JSON.stringify(respuesta));
        res.send(respuesta);
    });
    
});//elServidor.post

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  
 *       Funcion que maneja las peticiones GET /idiomas/       *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
elServidor.get('/idiomas/', function(req, res){
    console.log('Recibi peticion GET /idiomas/ .');
    
    laLogica.getIdiomasDisponibles(function(idiomasDisponibles){
        console.log("server/idiomasDisponibles: "+JSON.stringify(idiomasDisponibles));
        res.send(JSON.stringify(idiomasDisponibles));
    });
    
});//elServidor.get

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  
 *   Funcion que maneja las peticiones POST  /transpondedores/ *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
elServidor.post('/huellas/', function(req, res){
    console.log('Recibi peticion POST /huellas/');
    
    laLogica.crearHuella(JSON.parse(req.body), function(respuesta){
        console.log("respuesta: "+JSON.stringify(respuesta));
        res.send(JSON.stringify(respuesta));
    });
    
});//elServidor.post

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  
 *       Funcion que maneja las peticiones GET /huellas/       *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
elServidor.get('/huellas/', function(req, res){
    console.log('Recibi peticion GET /huellas/ .');
    
    var respuesta = []; 
    laLogica.getHuellas(function(huellasPendientes, huellasGuardadas){
        respuesta[0] = huellasPendientes;
        respuesta[1] = huellasGuardadas;
        console.log("server/huellas: "+JSON.stringify(respuesta));
        res.send(JSON.stringify(respuesta));
    });
    
});//elServidor.get

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  
 *   Funcion que maneja las peticiones POST  /dibujo/          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
elServidor.post('/dibujo/', function(req, res){
    console.log('Recibi peticion POST /dibujo/');
    
    laLogica.getDibujo(JSON.parse(req.body), function(respuesta){
        console.log("respuesta: "+JSON.stringify(respuesta));
        res.send(JSON.stringify(respuesta));
    });
    
});//elServidor.post

elServidor.post('/borrar/', function(req, res){
    console.log("Recibi peticion post /borrar/");
    
    laLogica.borrarDibujo(JSON.parse(req.body), function(respuesta){
        res.send(JSON.stringify(respuesta));
    });
});

//El servidor queda a la escucha del puerto 8080
elServidor.listen(8080);
console.log('Eschuchando puerto 8080...');