function enviarConsulta(){
    /*
     * Esta función envía los datos introducidos por el usuario al servidor
     */
    console.log("enviarConsulta()");
    
    // Comprobamos que sólo hay dibujada una ruta para evitar ambigüedades
    if(draw.getAll().features.length > 1){
        alert("Por favor, dibuje sólo una ruta. Pulse Refresh o recargue la página e inténtelo de nuevo.");
    }else{
        // Rellenamos el objeto JSON con el que trasladaremos los parametros al servidor
        var datosDeConsulta = 	{"ruta":    draw.getAll().features[0].geometry.coordinates, 
                                "idiomas": [document.getElementById('idioma1').value, 
                                            document.getElementById('idioma2').value, 
                                            document.getElementById('idioma3').value, 
                                            document.getElementById('idioma4').value,
                                           ],
                                           
                                 "antena": document.getElementById('antena').value};
        // Utilizamos la biblioteca XMLHttpRequest para implementar los mensajes REST
        var xmlHttp = new XMLHttpRequest();
        xmlHttp.open( "POST", 'http://127.0.0.1:8080/canales/', true ); // false for synchronous request
        xmlHttp.send( JSON.stringify(datosDeConsulta) );
        
        // Esta función detecta cuándo hemos recibido la respuesta del servidor
        xmlHttp.onreadystatechange = function(){
            if(xmlHttp.readyState == 4 && xmlHttp.status == 200){
                    //alert(xmlHttp.responseText);
                    
                    // Llamamos a crearTabla() para mostrar la respuesta del servidor al usuario
                    crearTabla(xmlHttp.responseText);
            }
        }
    }
}

function crearTabla(stringJSON){
    /*
     * Esta función genera una tabla con filtros y la rellena con los datos recibidos del servidor
     */
    console.log("crearTabla()");
    
    // El intercambio de información servidor-cliente lo hacemos en string para evitar problemas de compatibilidad,
    // pero necesitamos trabajar con las variables como objeto para emplear sus métodos
    objetoJSON = JSON.parse(stringJSON);
    // Buscamos el elemento del documento HTML donde vamos a desplegar la tabla
    document.getElementById('espacioDeTabla').innerHTML = '';
    document.getElementById('espacioDeTabla').innerHTML = '<table style="width:70%" border="1" id="tabla"><tr id="titulos"><th>Sel.</th><th>Nombre</th><th>Icono</th><th>Idiomas</th><th>Satélite             </th><th>Encoding</th><th>Encrypt (FTA)</th><th>Tipo</th><th>Frec.</th><th>Pol.</th><th>TID</th> </tr> </table>';
    
    // Recorremos la respuesta del servidor para formar las filas de la tabla
    for(var i = 0; i <= (objetoJSON.payload.length-1); i++){
        var fila = document.createElement("tr");
        
        // Recorremos las 11 columnas que queremos mostrar para rellenar las celdas
        for(var j = 0; j <= 10; j++){
            var celda = document.createElement("td");
            
            // Según el índice de la celda en la fila introducimos el atributo correspondiente
            switch(j){
                case 0: var contenidoCelda = document.createElement('input');
                        contenidoCelda.type = "button"; // Este botón nos permite elegir el canal que será el patrón que filtre la tabla según sus parámetros (transpondedor y polaridad)
                        contenidoCelda.setAttribute("onclick", "excluirCanalesIncompatibles("+i+")"); break;
                case 1: var contenidoCelda = document.createTextNode(objetoJSON.payload[i].nombre); break;
                case 2: var contenidoCelda = document.createTextNode(objetoJSON.payload[i].URL);  break;        // Hay que pedir el icono
                case 3: var contenidoCelda = document.createTextNode(objetoJSON.payload[i].idiomas);  break;    
                case 4: var contenidoCelda = document.createTextNode(objetoJSON.payload[i].nombre_sat);  break;
                case 5: var contenidoCelda = document.createTextNode(objetoJSON.payload[i].codificacion);  break;
                case 6: var contenidoCelda = document.createTextNode(objetoJSON.payload[i].encriptacion);  break;
                case 7: var contenidoCelda = document.createTextNode(objetoJSON.payload[i].tipo);  break;
                case 8: var contenidoCelda = document.createTextNode(objetoJSON.payload[i].frecuencia);  break;
                case 9: var contenidoCelda = document.createTextNode(objetoJSON.payload[i].polaridad);  break;
                case 10: var contenidoCelda = document.createTextNode(objetoJSON.payload[i].TID);  break;
                case 11: var contenidoCelda = document.createTextNode(objetoJSON.payload[i].SymbolRate);  break;
                case 12: var contenidoCelda = document.createTextNode(objetoJSON.payload[i].tipo);  break;
                case 13: var contenidoCelda = document.createTextNode(objetoJSON.payload[i].URL);  break;
            }
            celda.appendChild(contenidoCelda);
            fila.appendChild(celda);
        }
        tabla.appendChild(fila);
    }
    
    // Establecemos el filtro en la tabla con la configuración que pasamos como JSON en el segundo parámetro de la llamada
    setFilterGrid("tabla", {
                            col_0:              "none",
                            col_1:              "none",
                            col_2:              "none",
                            col_3:              "select",
                            col_4:              "none",
                            col_5:              "none",
                            col_6:              "select",
                            col_7:              "select",
                            col_8:              "none",
                            col_9:              "select",
                            col_10:              "select",
                            /*col_11:              "none",
                            col_12:              "select",
                            col_13:              "none",*/
                            display_all_text: " [ Show all ] ",
                            sort_select:        true                                                
                        });
}

function excluirCanalesIncompatibles(id){
    /*
     * Esta función aplica a la tabla el filtro sobre el transpondedor y la polaridad que tenga el canal seleccionado
     * por el usuario como patrón.
     */
    console.log("excluirCanalesIncompatibles("+id+")");
    
    // Recogemos los valores de las celdas de interés de la fila seleccionada por el usuario
    var TID = tabla.rows[id+2].cells.item(9).innerHTML;
    var pol = tabla.rows[id+2].cells.item(10).innerHTML;
    //var encrypt = tabla.rows[id+2].cells.item(9).innerHTML;

    // De la documentación oficial del filtro sabemos que el nombre del objeto TF que contiene las 
    // funciones sobre el filtro es "tf_"+<id de la tabla>
    tf_tabla.SetFilterValue(9, TID);
    tf_tabla.SetFilterValue(10, pol);
    //tf_tabla.SetFilterValue(9, encrypt);
    
    // Hacemos efectivo el filtro que acabamos de determinar
    tf_tabla.Filter();
}

function getIdiomas(){
    /*
     * Necesitamos saber qué idiomas hay disponibles para dárselos a elegir al usuario en los 
     * desplegables. Esta función responde a esa necesidad.
     */
    console.log("getHuellasPendientes()");
    
    // Utilizamos la biblioteca XMLHttpRequest para implementar los mensajes REST
    var xmlHttp = new XMLHttpRequest();
    xmlHttp.open(	"GET", 'http://127.0.0.1:8080/idiomas/', true ); //false for synchronus
    xmlHttp.send();
    
    // Detectamos la respuesta del servidor
    xmlHttp.onreadystatechange = function(){
        if(xmlHttp.readyState == 4 && xmlHttp.status == 200){
            // Trasladamos la respuesta del servidor a los deplegables sobre los que elegirá el usuario
            var idiomasDisponibles = JSON.parse(xmlHttp.responseText);
            cargarDesplegables(idiomasDisponibles);
        }
    }
}

function cargarDesplegables(idiomasDisponibles){
    /*
     * Esta función indtroduce el código HTML necesario para que los desplegables donde elige
     * el usuario el idioma que le interesa ofrezcan aquellos que el servidor ofrece.
     */
    console.log("cargarDesplegables()");
    
    // Buscamos los elementos que tenemos que rellenar
    var desplegable1 = document.getElementById('idioma1');
    var desplegable2 = document.getElementById('idioma2');
    var desplegable3 = document.getElementById('idioma3');
    var desplegable4 = document.getElementById('idioma4');
    
    // Recorremos la respuesta del servidor mientras introducimos el código HTML correspondiente
    for(var i = 0; i <= idiomasDisponibles.length-1; i++){
        desplegable1.innerHTML += "<option value='"+idiomasDisponibles[i].cod_iso+"'>"+idiomasDisponibles[i].cod_iso+"</option>";
        desplegable2.innerHTML += "<option value='"+idiomasDisponibles[i].cod_iso+"'>"+idiomasDisponibles[i].cod_iso+"</option>";
        desplegable3.innerHTML += "<option value='"+idiomasDisponibles[i].cod_iso+"'>"+idiomasDisponibles[i].cod_iso+"</option>";
        desplegable4.innerHTML += "<option value='"+idiomasDisponibles[i].cod_iso+"'>"+idiomasDisponibles[i].cod_iso+"</option>";

    }
}
